import { GraphQLResolveInfo } from "graphql";

export interface ResolverFn<Root, Args, Ctx, Payload> {
  (root: Root, args: Args, ctx: Ctx, info: GraphQLResolveInfo):
    | Payload
    | Promise<Payload>;
}

export interface ITypes {
  Context: any;
  TODO_STATE: any;
  User: any;
  QueryRoot: any;
  EmployeeRoot: any;
  FreelancerRoot: any;
  TodoRoot: any;
}

export namespace IQuery {
  export interface ArgsTodos {
    completed: boolean;
  }

  export type TodosResolver<T extends ITypes> = ResolverFn<
    T["QueryRoot"],
    ArgsTodos,
    T["Context"],
    T["TodoRoot"][]
  >;

  export interface Resolver<T extends ITypes> {
    todos: TodosResolver<T>;
  }
}

export namespace IEmployee {
  export type IdResolver<T extends ITypes> = ResolverFn<
    T["EmployeeRoot"],
    {},
    T["Context"],
    string
  >;

  export type NameResolver<T extends ITypes> = ResolverFn<
    T["EmployeeRoot"],
    {},
    T["Context"],
    string
  >;

  export interface Resolver<T extends ITypes> {
    id: IdResolver<T>;
    name: NameResolver<T>;
  }
}

export namespace IFreelancer {
  export type IdResolver<T extends ITypes> = ResolverFn<
    T["FreelancerRoot"],
    {},
    T["Context"],
    string
  >;

  export type NameResolver<T extends ITypes> = ResolverFn<
    T["FreelancerRoot"],
    {},
    T["Context"],
    string
  >;

  export type Task_costResolver<T extends ITypes> = ResolverFn<
    T["FreelancerRoot"],
    {},
    T["Context"],
    number
  >;

  export interface Resolver<T extends ITypes> {
    id: IdResolver<T>;
    name: NameResolver<T>;
    task_cost: Task_costResolver<T>;
  }
}

export namespace ITodo {
  export type IdResolver<T extends ITypes> = ResolverFn<
    T["TodoRoot"],
    {},
    T["Context"],
    string
  >;

  export type TextResolver<T extends ITypes> = ResolverFn<
    T["TodoRoot"],
    {},
    T["Context"],
    string
  >;

  export type CompletedResolver<T extends ITypes> = ResolverFn<
    T["TodoRoot"],
    {},
    T["Context"],
    boolean
  >;

  export type StateResolver<T extends ITypes> = ResolverFn<
    T["TodoRoot"],
    {},
    T["Context"],
    T["TODO_STATE"]
  >;

  export type Assigned_toResolver<T extends ITypes> = ResolverFn<
    T["TodoRoot"],
    {},
    T["Context"],
    T["User"][]
  >;

  export interface Resolver<T extends ITypes> {
    id: IdResolver<T>;
    text: TextResolver<T>;
    completed: CompletedResolver<T>;
    state: StateResolver<T>;
    assigned_to: Assigned_toResolver<T>;
  }
}

export interface IResolvers<T extends ITypes> {
  Query: IQuery.Resolver<T>;
  Employee: IEmployee.Resolver<T>;
  Freelancer: IFreelancer.Resolver<T>;
  Todo: ITodo.Resolver<T>;
}
